ClosetGnomeFishing = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0", "AceHook-2.1")

local bonus = AceLibrary("ItemBonusLib-1.0")
local deformat = AceLibrary("Deformat-2.0")
local L = AceLibrary("AceLocale-2.2"):new("ClosetGnomeFishing")

local slotmap = {
   ["INVTYPE_CHEST"] = { "Chest" },
   ["INVTYPE_CLOAK"] = { "Back" },
   ["INVTYPE_FEET"] = { "Feet" },
   ["INVTYPE_FINGER"] = { "Finger0", "Finger1" },
   ["INVTYPE_HAND"] = { "Hands" },
   ["INVTYPE_HEAD"] = { "Head" },
   ["INVTYPE_HOLDABLE"] = { "MainHand", "SecondaryHand" },
   ["INVTYPE_LEGS"] = { "Legs" },
   ["INVTYPE_NECK"] = { "Neck" },
   ["INVTYPE_RANGED"] = { "Ranged" },
   ["INVTYPE_RELIC"] = { "Ranged" },
   ["INVTYPE_ROBE"] = { "Chest" },
   ["INVTYPE_SHIELD"] = { "SecondaryHand" },
   ["INVTYPE_SHOULDER"] = { "Shoulder" },
   ["INVTYPE_TRINKET"] = { "Trinket0", "Trinket1" },
   ["INVTYPE_WAIST"] = { "Waist" },
   ["INVTYPE_WEAPON"] =  { "MainHand", "SecondaryHand" },
   ["INVTYPE_WEAPONMAINHAND"] = { "MainHand" },
   ["INVTYPE_WEAPONOFFHAND"] = { "SecondaryHand" },
   ["INVTYPE_2HWEAPON"] = { "MainHand" },
   ["INVTYPE_WRIST"] = { "Wrist" },
};

local mainhand, offhand

local FishingSpellID, FishingSkillName
-- support finding the fishing skill
local function FindSpellID(thisone)
   local id = 1
   local spellTexture = GetSpellTexture(id, BOOKTYPE_SPELL)
   while (spellTexture) do
      if (spellTexture and spellTexture == thisone) then
                return id
      end
      id = id + 1
      spellTexture = GetSpellTexture(id, BOOKTYPE_SPELL)
   end
   return nil
end

local function GetFishingSpellID()
   local FISHINGTEXTURE = "Interface\\Icons\\Trade_Fishing"
   if ( not FishingSpellID or not FishingSkillName) then
      FishingSpellID = FindSpellID(FISHINGTEXTURE)
   end
   if ( FishingSpellID and not FishingSkillName ) then
      FishingSkillName = GetSpellName(FishingSpellID, BOOKTYPE_SPELL)
   end
   return FishingSpellID, FishingSkillName
end

function ClosetGnomeFishing:FindFishingSet()
   local possibles = {}
   local numSlots = 0
   -- check each of the bags on the player
   for bag=0, NUM_BAG_FRAMES do
      -- get the number of slots in the bag (0 if no bag)
      numSlots = GetContainerNumSlots(bag)
      if (numSlots > 0) then
         -- check each slot in the bag
         for slot=1, numSlots do
            local link = GetContainerItemLink (bag, slot)
            if ( link ) then
               local info = bonus:ScanItemLink(link)
               if ( info ) then
                  possibles[link] = info.bonuses["FISHING"]
               end
            end
         end
      end
   end
   for slot=1,19 do
      local link = GetInventoryItemLink("player", slot)
      if ( link ) then
         local info = bonus:ScanItemLink(link)
         if ( info ) then
            possibles[link] = info.bonuses["FISHING"]
         end
      end
   end

   local slots = {}
   for link,points in pairs(possibles) do
      local _, _, _, _, _, _, _, _, loc, _ = GetItemInfo(link)
      local where = slotmap[loc]
      if ( where ) then
         local gotslot = false
         for _,slot in pairs(where) do
            if ( not gotslot and not slots[slot] ) then
               slots[slot] = { link = link, points = points }
               gotslot = true
            end
         end
         if ( not gotslot ) then
             for _,slot in pairs(where) do
               if ( not gotslot and slots[slot].points < points) then
                  slots[slot] = { link = link, points = points }
                  gotslot = true
               end
            end
         end
      end
      if ( "INVTYPE_2HWEAPON" == loc ) then
         slots[offhand] = { link = true, points = 10000 }
      end
   end

   local print = ClosetGnome.Print
   ClosetGnome.Print = function() return end
   local name = L["Fishing"]
   if ( ClosetGnome:HasSet(name) ) then
      ClosetGnome:DeleteSet(name, true)
   end
   for slot,info in pairs(slots) do
      if ( not ClosetGnome.db.char.set[name] ) then ClosetGnome.db.char.set[name] = {} end
      ClosetGnome.db.char.set[name][slot] = info.link
   end
   ClosetGnome:UpdateSetMenu()
   ClosetGnome:UpdateDisplay()
   ClosetGnome.Print = print
end

function ClosetGnomeFishing:OnInitialize()
   local s = {}
   for loc,where in pairs(slotmap) do
      local w = {}
      for _,slotname in pairs(where) do
        local slot = GetInventorySlotInfo(slotname.."Slot")
        tinsert(w, slot)
      end
      s[loc] = w
   end
   slotmap = s

   mainhand = GetInventorySlotInfo("MainHandSlot")
   offhand = GetInventorySlotInfo("SecondaryHandSlot")
end

function ClosetGnomeFishing:OnEnable()
   self:RegisterEvent("SPELLS_CHANGED")
   self:RegisterEvent("UI_ERROR_MESSAGE")

   ClosetGnome:UpdateSetMenu()

   self:FindFishingSet()
end

-------------------------------------------------------------------------------
-- Events
-------------------------------------------------------------------------------
function ClosetGnomeFishing:UI_ERROR_MESSAGE(msg)
   local pole = deformat(msg, SPELL_FAILED_EQUIPPED_ITEM_CLASS)
   if pole and pole == L["Fishing Pole"] then
      self:Equip()
   end
end

function ClosetGnomeFishing:SPELLS_CHANGED()
   FishingSpellID = nil
   self:FindFishingSet()
end

function ClosetGnomeFishing:Equip()
   if ClosetGnome:HasSet(L["Fishing"]) then
      ClosetGnome:WearSet(L["Fishing"])
   end
end
